package admin

import (
	"github.com/graphql-go/graphql"
	q "gitlab.okta-solutions.com/mashroom/backend/common/api/generic/query"
	"gitlab.okta-solutions.com/mashroom/backend/common/graphql/util"
	"gitlab.okta-solutions.com/mashroom/backend/gateway/impl/connections"
	"gitlab.okta-solutions.com/mashroom/backend/gateway/impl/schema"
	offer "gitlab.okta-solutions.com/mashroom/backend/offer/api"
	property "gitlab.okta-solutions.com/mashroom/backend/property/api"
)

type allParamsPagination struct {
	Page    int32 `json:"page,omitempty"`
	PerPage int32 `json:"perPage,omitempty"`
}

type allParams struct {
	Pagination allParamsPagination    `json:"pagination,omitempty"`
	Sort       []*q.QueryParamsSort   `json:"sort,omitempty"`
	Filter     []*q.QueryParamsFilter `json:"filter,omitempty"`
}

type getParams struct {
	Id string `json:"id"`
}

func init() {
	queryFields := graphql.Fields{}
	mutationFields := graphql.Fields{}

	initUser(&queryFields, &mutationFields)
	initAdmin(&queryFields, &mutationFields)
	initProperty(&queryFields, &mutationFields)
	initOffer(&queryFields, &mutationFields)
	initDocument(&queryFields, &mutationFields)
	initRightmove(&queryFields, &mutationFields)
	initZoopla(&queryFields, &mutationFields)
	initContract(&queryFields, &mutationFields)
	initCardPayment(&queryFields, &mutationFields)
	initSignature(&queryFields, &mutationFields)

	query := graphql.NewObject(graphql.ObjectConfig{
		Name:        "AdminQuery",
		Description: "Admin query root",
		Fields:      queryFields,
	})

	schema.RootQueries["admin"] = &graphql.Field{
		Type: query,
		Resolve: func(p graphql.ResolveParams) (interface{}, error) {
			return query, nil
		},
	}

	mutation := graphql.NewObject(graphql.ObjectConfig{
		Name:        "AdminMutation",
		Description: "Admin mutation root",
		Fields:      mutationFields,
	})

	schema.RootMutations["admin"] = &graphql.Field{
		Type: mutation,
		Resolve: func(p graphql.ResolveParams) (interface{}, error) {
			return mutation, nil
		},
	}

	initOfferEnrichment()
}

func initOfferEnrichment() {
	util.BindOutput(offerData{}, "offer", util.EnrichmentMap{
		"viewing": &util.Enrichment{
			Output: func(bind util.BindType) graphql.Output {
				return util.BindOutput(&property.ViewRequest{}, "viewing", nil)
			},
			Resolver: func(p graphql.ResolveParams, oldvalue interface{}) (i interface{}, e error) {
				req, ok := p.Source.(*offer.Offer)
				if !ok || len(req.GetTenantsValue().GetValue()) < 1 {
					return &property.ViewRequest{}, nil
				}
				view, err := connections.PropertyService.GetView(p.Context, &property.GetViewRequest{
					User:     req.GetTenantsValue().GetValue()[0].GetUser(),
					Property: req.GetProperty(),
					Id:       req.GetViewing(),
				})
				if err != nil || view == nil || view.GetRequest() == nil {
					return &property.ViewRequest{}, nil
				}
				return view.GetRequest(), nil
			},
		},
		"property": &util.Enrichment{
			Output: func(bind util.BindType) graphql.Output {
				return util.BindOutput(&property.Property{}, "property", nil)
			},
			Resolver: func(p graphql.ResolveParams, oldvalue interface{}) (i interface{}, e error) {
				req, ok := p.Source.(*offer.Offer)
				if !ok {
					return &property.Property{}, nil
				}

				if req.GetProperty() == "" {
					return &property.Property{}, nil
				}

				resp, err := connections.PropertyService.Get(p.Context, &property.GetRequest{Id: req.GetProperty()})
				if err != nil || resp == nil || resp.GetProperty() == nil {
					return &property.Property{}, nil
				}
				return resp.GetProperty(), nil
			},
		},
	})
}
