package resolvers

import (
	"github.com/graphql-go/graphql"
	"gitlab.okta-solutions.com/mashroom/backend/common/graphql/util"
	"gitlab.okta-solutions.com/mashroom/backend/gateway/impl/connections"
	"gitlab.okta-solutions.com/mashroom/backend/gateway/impl/schema"
	"gitlab.okta-solutions.com/mashroom/backend/verification/api"
)

func init() {
	query := graphql.NewObject(graphql.ObjectConfig{
		Name:        "VerificationQuery",
		Description: "Verification query root",
		Fields: graphql.Fields{
			"value": &graphql.Field{
				Description: "Validates distinct value",
				Args:        util.BindArguments(verification.VerifyValueRequest{}),
				Resolve:     VerificationQueryValueResolver,
				Type:        util.BindOutput(verification.VerifyValueResponse{}, "Value verification result", nil),
			},
			"batch": &graphql.Field{
				Description: "Validates batch of values",
				Args:        util.BindArguments(verification.VerifyBatchRequest{}),
				Resolve:     VerificationQueryBatchResolver,
				Type:        util.BindOutput(verification.VerifyBatchResponse{}, "Batch verification result", nil),
			},
		},
	})

	schema.RootQueries["verification"] = &graphql.Field{
		Type: query,
		Resolve: func(p graphql.ResolveParams) (interface{}, error) {
			return query, nil
		},
	}
}

func VerificationQueryValueResolver(params graphql.ResolveParams) (interface{}, error) {
	request := &verification.VerifyValueRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	return connections.VerificationService.VerifyValue(params.Context, request)
}

func VerificationQueryBatchResolver(params graphql.ResolveParams) (interface{}, error) {
	request := &verification.VerifyBatchRequest{}
	if err := util.UnmarshalArguments(params.Args, request); err != nil {
		return nil, err
	}
	return connections.VerificationService.VerifyBatch(params.Context, request)
}
