package rest

import (
	"fmt"
	"gitlab.okta-solutions.com/mashroom/backend/common/generic/log"
	"gitlab.okta-solutions.com/mashroom/backend/gateway/impl/connections"
	"gitlab.okta-solutions.com/mashroom/backend/gateway/impl/resolvers"
	"gitlab.okta-solutions.com/mashroom/backend/offer/api"
	"gitlab.okta-solutions.com/mashroom/backend/payment/api"
	"net/http"
	"strings"
)

const errorUrl = "https://%s/profile/tenant/overview"

func onError(w http.ResponseWriter, err error) {
	log.WithError(err).Error("TokenioHandler")
	w.WriteHeader(http.StatusInternalServerError)
	w.Header().Set("Location", fmt.Sprintf(errorUrl, resolvers.SiteUrl))
	_, _ = w.Write(([]byte)(err.Error()))
}

func TokenioHandler(w http.ResponseWriter, r *http.Request) {
	c, err := r.Cookie("csrf_tokenio")
	if err != nil {
		onError(w, err)
		return
	}

	// check decline error
	if len(r.URL.Query()) > 0 {
		for k, v := range r.URL.Query() {
			if strings.ToLower(k) == "error_description" && len(v) > 0 && strings.ToLower(v[0]) == "terms declined" {
				tresp, err := connections.OfferService.PaymentCsrfToken(r.Context(), &offer.PaymentCsrfTokenRequest{
					CsrfToken: c.Value,
				})
				if err != nil {
					onError(w, err)
					return
				}

				w.Header().Set("Location", fmt.Sprintf("https://%s/profile/offers/edit?id=%s&step=deposit", resolvers.SiteUrl, tresp.Offer))
				w.WriteHeader(http.StatusFound)
				return
			}
		}
	}

	cburl := fmt.Sprintf("https://%s%s", resolvers.SiteUrl, r.URL.String())
	log.WithField("CallbackUrl", cburl).WithField("CsrfToken", c.Value).Info()
	resp, err := connections.PaymentService.RedeemToken(r.Context(), &payment.RedeemTokenRequest{
		CallbackUrl: cburl,
		CsrfToken:   c.Value,
	})
	if err != nil {
		onError(w, err)
		return
	}
	log.WithField("Id", resp.GetRefId()).WithField("Status", resp.GetStatus()).WithField("TransferId", resp.GetTransferId()).Info()
	oresp, err := connections.OfferService.PaymentResult(r.Context(), &offer.PaymentResultRequest{
		Id:         resp.GetRefId(),
		Status:     resp.GetStatus(),
		TransferId: resp.GetTransferId(),
	})
	if err != nil {
		onError(w, err)
		return
	}
	log.WithField("Offer", oresp.GetOffer()).WithField("Ok", oresp.GetOk())

	w.Header().Set("Location", fmt.Sprintf("https://%s/profile/tenant/offers/overview", resolvers.SiteUrl))
	w.WriteHeader(http.StatusFound)
}
