package rest

import (
	"context"
	"fmt"
	"gitlab.okta-solutions.com/mashroom/backend/gateway/impl/connections"
	"gitlab.okta-solutions.com/mashroom/backend/twilio/api"
	"io/ioutil"
	"net/http"
)

func TwilioIvrHandler(w http.ResponseWriter, r *http.Request) {
	body, _ := ioutil.ReadAll(r.Body)

	if len(body) == 0 && r.URL != nil {
		body = []byte(r.URL.RawQuery)
	}
	if len(body) == 0 {
		http.Error(w, "Forbidden for empty request", http.StatusForbidden)
		return
	}

	resp, err := connections.TwilioService.IVRCallbackHandler(context.Background(), &twilio.IVRCallbackHandlerRequest{
		Data: body,
	})
	if err != nil {
		fmt.Printf("TwilioIvrHandler err %v\n", err)
		http.Error(w, err.Error(), http.StatusForbidden)
		return
	}

	w.Header().Set("Content-Type", "application/xml")
	//w.WriteHeader(http.StatusOK)
	_, _ = w.Write(resp.Data)
	return
}

func TwilioFileHandler(writer http.ResponseWriter, request *http.Request) {
	resp, err := connections.TwilioService.IVRFileHandler(context.Background(), &twilio.IVRFileHandlerRequest{File: request.URL.Path})
	if err != nil {
		fmt.Printf("TwilioFileHandler err %v\n", err)
		http.Error(writer, err.Error(), 500)
		return
	}

	writer.Header().Set("Content-Disposition", "attachment; filename="+resp.File)
	writer.Header().Set("Content-Type", resp.Mime)
	writer.Header().Set("Content-Length", fmt.Sprintf("%d", len(resp.Data)))
	//writer.WriteHeader(http.StatusOK)
	_, _ = writer.Write(resp.Data)
	return
}
